/*
 * https://github.com/mattma/angular-cli-generator
 *
 * Copyright (c) 2017 Matt Ma
 * Licensed under the MIT license.
 */

'use strict';

const program = require('commander');
const generate = require('./generate');
const pkg = require('../../package.json');

program
  .version('angular-cli-generator: ' + pkg.version)
  .usage(
    '[command] [options]\n\n  Command-Specific Help\n\n    ngg [command] --help'
  );

program
  .command('g')
  .alias('generate')
  .description(
    '@description: Generate folders or files that works compatible with angular cli'
  )
  .usage(generateCommandInfo())
  .option('-p, --project', 'An optional project namespace')
  // WIP
  // .option('-s, --selector-prefix', 'component selector prefix')
  .action(generate);

// must be before .parse() since node's emit() is immediate
program.on('--help', helpCommandInfo);

program.parse(process.argv);

function generateCommandInfo() {
  var ret =
    '<type> <name>' +
    '\n\n    ngg g route <ROUTE_NAME>' +
    '\n        @example: ngg g route post -p rocks   # generated: project/hero/src/app/post/' +
    '\n        @example: ngg g route my-blog         # generated: src/app/my-blog/';

  return ret;
}

function helpCommandInfo() {
  console.log('');
  console.log('');
  console.log('  Examples:');
  console.log('');

  console.log(
    '    $ ngg generate <type> <name>' +
      "\n       @params {string} <type> - 'route'," +
      '\n       @params {string} <name> - any string or dash separated string',
    '\n       @hint  ngg g --help  # command for more detail'
  );
  console.log('');
}
